/****************************************************************************
* 
* EBus.c - Allgemeine Messages verarbeiten 
*
****************************************************************************/

#include <Arduino.h>
#include "EBusAna.h"
#include "Global.h"
#include "Wolf.h"

/****************************************************************************
* globale Daten
****************************************************************************/

struct HEIZ_INFO HeizInfo[ NumPara ] =
  {
  { "Brenner_Leistung",  0, _UBYTE   },
  { "Heiz_Ruekl_Temp",   0, _UBYTE   },
  { "Status1",           0, _STATUSW },
  { "Kessel",            0, _UBYTE   },
  { "Puffer",            0, _UBYTE   },
  { "Status",            0, _STATUS  },
  { "Soll_Kessel",       0, _MESS    },
  { "Soll_Puffer",       0, _MESS    },
  { "Uhrzeit",           0, _TIME    },
  { "Tag",               0, _TAG     },
  { "Aussen_Temp",       0, _MESS    },
  { "Status2",           0, _STATUSB },
  { "Mittel_Aussen",     0, _MESS    },
  { "Status3",           0, _STATUSB },
  WolfSpezData
  };

/****************************************************************************
* Funktionen
****************************************************************************/
/*--------------------------------------------------------------------------
* DataFromMes - Daten aus Messages lesen 
* Parameter: pBuf - Zeiger auf Puffer
*            len - Zahl der Bytes im Puffer
* Ergebnis:  keins
*--------------------------------------------------------------------------*/

void DataFromMes( uint8_t *pBuf, uint8_t len )

  {
  uint16_t Code;
  
  if( pBuf[2] == 0x50 )         /* Wolf spezifische Message */
    {
    DataFromWolfMes( pBuf, len );
    return;
    }

  Code = ((uint16_t)pBuf[2] << 8) | pBuf[3];

// Brenner an Alle 05 -> Brenner Steuerbefehl, 03 -> Betriebsdaten des Feuerungsautomaten
  if( (Code == 0x0503) && (len >= 13) )
    {
    if( pBuf[0] == 0x03 )
      {
      HeizInfo[ _BrennLeist     ].Data.UB = pBuf[ 8 ];  
      HeizInfo[ _HeizRuecklTemp ].Data.UB = pBuf[ 10 ];  
      }
    else if( pBuf[0] == 0xF1 )
      {
      HeizInfo[ _Status1    ].Data.UW = *((uint16_t *)&pBuf[ 6 ]);  
//    HeizInfo[ _AussenTemp ].Data.W  = pBuf[ 12 ];    /* gibt es an anderer Stelle mit Nachkomma */
      }
    HeizInfo[ _KesselTemp ].Data.UB = pBuf[ 9 ] / 2;
    HeizInfo[ _PufferTemp ].Data.UB = pBuf[ 11 ];

    if( EEPromData.Mode == MODE_HEX_INFO )
      {
      if( pBuf[0] == 0x03 )
        {
        PrintText( _BrennLeist );
        PrintText( _HeizRuecklTemp );
        }
      else if( pBuf[0] == 0xF1 )
        {
//      PrintText( _AussenTemp );
        PrintText( _Status1 );
        }
      PrintText( _KesselTemp );
      PrintText( _PufferTemp );
      PrintNewLine();
      }
    }

// BM an Brenner: 05 -> Brenner Steuerbefehl, 07 -> Betriebsart Regler
  else if( (Code == 0x0507) && (len >= 14) )
    {
    HeizInfo[ _Status      ].Data.S.Kessel = pBuf[ 5 ] & 0x0F;
    HeizInfo[ _SKesselTemp ].Data.W         =  16 * *((uint16_t *)&pBuf[ 7 ]);
    HeizInfo[ _SPufferTemp ].Data.W         = (uint16_t)128 * pBuf[ 12 ];

    if( EEPromData.Mode == MODE_HEX_INFO )
      {
      PrintText( _Status );
      PrintText( _SKesselTemp );
      PrintText( _SPufferTemp );
      PrintNewLine();
      }
    }

// BM an Alle: 07 -> Systemdatenbefehle, 00 -> Datum Uhrzeit
  else if( (Code == 0x0700) && (len >= 14) )
    {
    HeizInfo[ _Uhrzeit ].Data.UW = *((uint16_t *)&pBuf[ 8 ]);
    HeizInfo[ _Tag     ].Data.UB = pBuf[ 12 ];

    if( EEPromData.Mode == MODE_HEX_INFO )
      {
      PrintText( _Uhrzeit );
      PrintText( _Tag );
      PrintNewLine();
      }
    }

// Brenner an alle (Antwort auf vorhergehende Nachricht)
// oder BM an Brenner
  else if( (Code == 0x0800) && (len >= 13) && (pBuf[0] != 0x03) )
    {
    HeizInfo[ _SKesselTemp ].Data.W = *((int16_t *)&pBuf[ 5 ]);
    if( pBuf[0] == 0x10 )
      {
      HeizInfo[ _AussenTemp  ].Data.W = *((int16_t *)&pBuf[ 7 ]);
      HeizInfo[ _Status2     ].Data.UB = pBuf[ 10 ]; 
      }
    else if( pBuf[0] == 0xF1 )
      {
      HeizInfo[ _MAussenTemp ].Data.W  = *((int16_t *)&pBuf[ 7 ]);
      HeizInfo[ _Status3     ].Data.UB = pBuf[ 10 ]; 
      }
    HeizInfo[ _SPufferTemp ].Data.W = *((int16_t *)&pBuf[ 11 ]);

    if( EEPromData.Mode == MODE_HEX_INFO )
      {
      PrintText( _SKesselTemp );
      if( pBuf[0] == 0x10 )
        {
        PrintText( _AussenTemp );
        PrintText( _Status2 );
        }
      else if( pBuf[0] == 0xF1 )
        {
        PrintText( _MAussenTemp );
        PrintText( _Status3 );
        }
      PrintText( _SPufferTemp );
      PrintNewLine();
      }
    } 
  }

  
/*--------------------------------------------------------------------------
* CalcCRC - CSC Prüfsumme eines Datenpaketes berechnen
* Parameter: data - Wert
*            crc_init  - letzte Prüfsumme
*            GenPolynome - Polynom für die CRC, bei EBus 0x9B
* Ergebnis:  Prüfsumme
*--------------------------------------------------------------------------*/

uint8_t crc8_calc( uint8_t data, uint8_t crc_init, uint8_t GenPolynom )

  {
  uint8_t crc;
  uint8_t polynom;
  uint8_t i;

  crc = crc_init;
  for ( i = 0; i < 8; i++ )
    {
    polynom = crc & 0x80 ? GenPolynom : 0;
    crc = (crc & ~0x80) << 1;
    if( data & 0x80 )
      crc |= 1;
    crc = crc ^ polynom;
    data <<= 1;
    }
  return (crc);
  }

  
/*--------------------------------------------------------------------------
* CalcCheckSum - Checksumme berechnen
* Parameter: pdata - Zeiger auf Daten
*            len   - Zahl der Zeichen
* Ergebnis:  Prüfsumme
*--------------------------------------------------------------------------*/

uint8_t CalcCheckSum( uint8_t *pdata, uint8_t len )

  {
  uint8_t crc = 0;
  
  while( len-- != 0 )
    crc = crc8_calc( *(pdata++), crc, 0x9B );

  return( crc );
  }

