/****************************************************************************
* 
* EBusAna.h - Project Header file
*
****************************************************************************/

#ifndef EBUSANA_H
#define EBUSANA_H

/****************************************************************************
* Definitionen
****************************************************************************/

/*** Hier lassen sich funktionen auskommentieren ***/
#define SD_LOGGING 5  /* Logging auf SD-Karte, Angabe des CS Ports, auskommentiren wenn kein logging moeglich */
#define SD_MES_LOG    /* Alle nachrichten loggen */
#define AK_READ_MESS  /* Aktiv Messwerte abholen */

/*** für Heizinfo Type ***/
#define _UBYTE    0   /* 8 Bit Zahl ohne Vorzeichen */
#define _BYTE     1   /* 8 Bit Zahl mit Vorzeichen */
#define _UMESS    2   /* 8 Bit Nachkomma + 8 Bit Vorkomma ohne Vorzeichen */
#define _MESS     3   /* 8 Bit Nachkomma + 8 Bit Vorkomma mit Vorzeichen */
#define _TIME     4   /* Stunden / Minuten in BCD */
#define _BIG      5   /* 16 Bit Vorkomma (/1000) + 16 Bit Nachkomma */
#define _VBIG     6   /* 16 Bit Vorkomma (/1000) + 16 Bit Nachkomma */
#define _TAG      7   /* Wochetag (1=Montag) */
#define _STATUS   8   /* Einige Stati */
#define _STATUSW  9   /* Einige Stati */
#define _STATUSB  10   /* Einige Stati */
#define _MESS16   11  /* 4 Bit Nachkomma + 12 Bit Vorkomma mit Vorzeichen */
#define _MESS10   12  /* Wert mal 10 (eine Nachkommastelle) */

#define _BrennLeist     0
#define _HeizRuecklTemp 1
#define _Status1        2
#define _KesselTemp     3
#define _PufferTemp     4
#define _Status         5
#define _SKesselTemp    6
#define _SPufferTemp    7
#define _Uhrzeit        8
#define _Tag            9
#define _AussenTemp    10
#define _Status2       11
#define _MAussenTemp   12
#define _Status3       13
#define _EBUSAllMax    14
#ifdef AK_READ_MESS
#define _EBUSWolfMax   12
#else /* #ifdef AK_READ_MESS */
#define _EBUSWolfMax   9
#endif /* #ifdef AK_READ_MESS */

#define PORT_LedGr    13    /* LED Gruen an IO13 */  

#define COMMAND_PORT  Serial
#define EBUS_PORT     Serial1

#define MODE_DATA     0   /* Audgewähhlte Messwerte ausgeben */  
#define MODE_HEX      1   /* Messages in HEX ausgeben */
#define MODE_HEX_INFO 2   /* Messages in HEX und Erklaerung ausgeben */ 
#define MODE_SILENT   3   /* nichts ausgeben */

#define NumPara       (_EBUSAllMax + _EBUSWolfMax)
#define SEPARATOR_SDC	9    	   /* Serparator fue Ausgabe SD Karte*/
#define SEPARATOR_CCU ';'      /* Serparator fuer Ausgabe USB */

/****************************************************************************
* Neue Typen
****************************************************************************/

struct HEIZ_STATUS
  {
  unsigned Kessel:4;
  unsigned SKPumpe:1;
  };

union DATA_TYPE
  {
  uint8_t  B;             /*  8 Bit Wert */
  uint8_t  UB;            /*  8 Bit Wert */
  uint16_t W;             /* 16 Bit Wert */
  uint16_t UW;            /* 16 Bit Wert */
  void *   P;             /* Zeiger auf längere Daten */
  struct HEIZ_STATUS S;   /* Status */
  };
  
struct HEIZ_INFO
  {
  const char *Text;       /* Zeiger auf Text */
  union DATA_TYPE Data;   /* Zeiger auf Messwert */
  uint8_t Type;           /* Tuep des Messwertes */
  };

/*** Daten im EEPROM ***/
struct EEPROM_DATA
  {
  uint16_t Stamp;           /* Kennung */
  uint16_t ZyklusZeit;      /* Zeit in Ticks zwischen 2 Zyklischen Ausgaben */
  uint8_t  Mode;            /* Ausgabemodus */
  uint8_t  Enable[NumPara]; /* True = Wert ausgeben */
  uint8_t  ChkSum;          /* Chechsumme */
  };


/****************************************************************************
* Globale Variablen
****************************************************************************/

extern struct HEIZ_INFO HeizInfo[ NumPara ];
extern struct EEPROM_DATA EEPromData;
extern char Dezimalpunkt;

/****************************************************************************
* Funktionen
****************************************************************************/
/*** UartIO.cpp ***/
/*--------------------------------------------------------------------------*
* FlexPrint - Ausgabe von Puffer auf Serial oder SD
* Parameter: pBuf - Zeiger auf Puffer
* Ergebnis:  keines
*--------------------------------------------------------------------------*/
void FlexPrint( char *pBuf );

/*--------------------------------------------------------------------------*
* PrintNewLine - Zeilenumbruch ausgeben
* Parameter: keine
* Ergebnis:  keines
*--------------------------------------------------------------------------*/
void PrintNewLine( void );

/*--------------------------------------------------------------------------*
* PrintHex - Git einen Wert Hexadezimal aus
* Parameter: Data - Wert
*            len  - Zahl der Zeichen, ungeraden -> Leerzeichen anhängen
* Ergebnis:  keines
*--------------------------------------------------------------------------*/
void PrintHex( uint16_t Data, uint8_t len );

/*--------------------------------------------------------------------------
* PrintText - Gibt Text und Parameter aus
* Parameter: Nummer des Messwertes
* Ergebnis:  keins
*--------------------------------------------------------------------------*/
void PrintText( uint8_t Mess );

/*--------------------------------------------------------------------------*
* PrintData - Gibt einem Messwert aus
* Parameter: Data - Zeiger auf die Daten
*            Type - T�p der Daten
* Ergebnis:  keines
*--------------------------------------------------------------------------*/
void PrintData( union DATA_TYPE Data, uint8_t Type );

/*--------------------------------------------------------------------------
* PrintSelData - Gibt ausgewaehlte Parameter aus
* Parameter: Mask - 1 = Seriell, 2 = SD - Karte
*            Sep - Trennzeichen
* Ergebnis:  keins
*--------------------------------------------------------------------------*/
void PrintSelData( uint8_t Mask, char Sep );

/*--------------------------------------------------------------------------
* SaveSelData - Speichert ausgewählte Parameter in Logdatei
* Parameter: keine
* Ergebnis:  keins
*--------------------------------------------------------------------------*/
void SaveSelData( void );

/*--------------------------------------------------------------------------
* SaveSelData - Log Message auf SD-Karte schreiben
* Parameter: keine
* Ergebnis:  keins
*--------------------------------------------------------------------------*/
void DataLogMsg( void );

/*--------------------------------------------------------------------------
* PrintAllData - Gibt ausgew�hlte Parameter aus
* Parameter: keine
* Ergebnis:  keins
*--------------------------------------------------------------------------*/
void PrintAllData( void );

/*--------------------------------------------------------------------------
* GetHex - 16Bit Hexadezimal Wert einlesen
* Parameter: pBuf - Zeiger auf String
*            pVal - Zeiger auf Ergebnis
* Ergebnis:  pBuf - neuer Zeiger auf String
*--------------------------------------------------------------------------*/
char *GetHex( char *pBuf, uint16_t *pVal );

/*--------------------------------------------------------------------------
* GetDec - 16Bit Dezimal Wert einlesen
* Parameter: pBuf - Zeiger auf String
*            pVal - Zeiger auf Ergebnis
* Ergebnis:  pBuf - neuer Zeiger auf String
*--------------------------------------------------------------------------*/
char *GetDec( char *pBuf, int16_t *pVal );

/*** EBus.cpp ***/
/*--------------------------------------------------------------------------
* DataFromMes - Daten aus Messages lesen 
* Parameter: pBuf - Zeiger auf Puffer
*            len - Zahl der Bytes im Puffer
* Ergebnis:  keins
*--------------------------------------------------------------------------*/
void DataFromMes( uint8_t *pBuf, uint8_t len );

/*--------------------------------------------------------------------------
* CalcCRC - CSC Prüfsumme eines Datenpaketes berechnen
* Parameter: data - Wert
*            crc_init  - letzte Prüfsumme
*            GenPolynome - Polynom für die CRC, bei EBus 0x9B
* Ergebnis:  Prüfsumme
*--------------------------------------------------------------------------*/
uint8_t crc8_calc( uint8_t data, uint8_t crc_init, uint8_t GenPolynom );

/*--------------------------------------------------------------------------
* CalcCheckSum - Checksumme berechnen
* Parameter: pdata - Zeiger auf Daten
*            len   - Zahl der Zeichen
* Ergebnis:  Prüfsumme
*--------------------------------------------------------------------------*/
uint8_t CalcCheckSum( uint8_t *pdata, uint8_t len );

#endif /* #ifndef EBUSANA_H */
