#include <stdint.h>
#include "Timer.h"
#include "EBusAna.h"
#include "Global.h"
#include "Wolf.h"
#include <EEPROM.h>
#include <Arduino.h>
#ifdef SD_LOGGING
#include <SPI.h>
#include <SD.h>
#endif /* #ifdef SD_LOGGING */
#include <avr/pgmspace.h>

/****************************************************************************
* Definitionen
****************************************************************************/

#define MAX_READ_MES  4U
#define READ_MES_LEN  8U
//#define TEST
#define SD_MES_LOG

/****************************************************************************
* Globale Variablen
****************************************************************************/

/*** Daten im EEPROM ***/
const PROGMEM struct EEPROM_DATA DefEEProm =
  {
  0xFACE,                /* Kennung */
  60000L/MSEC_PER_TICKS, /* Zeit in Ticks zwischen 2 Zyklischen Ausgaben */
  MODE_HEX_INFO,         /* Ausgabemodus */
  { 3, 3, 2, 3, 3, 3, 3, 3,       /* True = Wert ausgeben */
    3, 3, 3, 2, 3, 2,
    WolfSpezEnab },
  0
  };

struct EEPROM_DATA EEPromData;

#define EBUS_REC_BUF_SIZE 45U                    /* Min = 5 + 16 + 3 + 16 + 2 */
#define EBUS_TRA_BUF_SIZE 25U                    /* Min = 5 + 16 + 1 */
#define CMD_BUF_SIZE  40                         /* Groesse des Kommandopuffers */

#ifdef SD_LOGGING
File logFile;
#endif /* #ifdef SD_LOGGING */

static uint8_t Buffer[ EBUS_REC_BUF_SIZE ];      /* Message Buffer EBus */
static char    CmdBuf[ CMD_BUF_SIZE ];           /* Puffer für Commandos über USB */
static uint8_t BufPos = 0;                       /* Zeiger auf Puffer */ 
static uint8_t uc_crc = 0;                       /* Berechnete CRC */
static uint8_t doSend = 0;                       /* Vorgefertigte Message senden */
static uint8_t doSendAuto   = 0;                 /* Automatische Message senden */
static uint8_t SendAckOnBus = 0;                 /* 0 = nichts tun, 1 = 0x00 nach Nachricht senden, 2 = 0x00 + 0xAA nach Nachricht senden */
static uint8_t MesEnd;                           /* Errechnetes Ende der Message */
static uint8_t MesAntEnd;                        /* Errechnetes Ende der Message incl. Antwort */
static bool    RecErr = false;                   /* true = Fehler beim Empfang */
static uint8_t ReseveBus = 0;                    /* Bus nach einer Kollision nicht benutzen */
static uint8_t SendMsg[MAX_SPECMES_LEN+5];
                                                 /* Leer Message zum Abfragen von Werten */
const PROGMEM uint8_t SendMsg2[] = { 0xFF, 0x35, 0x50, 0x22, 0x03, 0x00, 0x00, 0x00 };

#if 0                                            /* Original Message um zwischen Standby / Zeitbetrieb / Heizen / Absenken / Warmwasser umzuschalten */
const PROGMEM uint8_t ModMsg[ 5 ][ 14 ] =
  {
    { 0xFF, 0x30, 0x50, 0x23, 0x09, 0x84, 0x12, 0x01, 0x01, 0x00, 0x5D, 0x01, 0x00, 0x00 }, /* Standby */
    { 0xFF, 0x30, 0x50, 0x23, 0x09, 0x68, 0x12, 0x01, 0x02, 0x00, 0x5D, 0x01, 0x00, 0x00 }, /* Zeitbetrieb */
    { 0xFF, 0x30, 0x50, 0x23, 0x09, 0xF8, 0x12, 0x01, 0x03, 0x00, 0x5D, 0x01, 0x00, 0x00 }, /* Heizen */
    { 0xFF, 0x30, 0x50, 0x23, 0x09, 0xEC, 0x12, 0x01, 0x04, 0x00, 0x5D, 0x01, 0x00, 0x00 }, /* Absenken */
    { 0xFF, 0x30, 0x50, 0x23, 0x09, 0x7C, 0x12, 0x01, 0x05, 0x00, 0x5D, 0x01, 0x00, 0x00 }, /* Warmwasser */
  };
#endif

#ifdef AK_READ_MESS
                                                 /* Mit diesen Messages werden Messwerte, die nicht automatisch über den Bus laufen abgefragt */
const PROGMEM uint8_t AutoMsg[ MAX_READ_MES ][ READ_MES_LEN ] = 
  {
    { 0xFF, 0x51, 0x50, 0x22, 0x03, 0x32, 0x29, 0x0A }, /* Messwert Temperatur des Pufferspeichers am Mischermodul (Mitte Speicher) abfragen */
    { 0xFF, 0x51, 0x50, 0x22, 0x03, 0xD6, 0x2A, 0x0A }, /* Messwert Rücklauf Mischer abfragen */
    { 0xFF, 0x76, 0x50, 0x22, 0x03, 0x8A, 0x2B, 0x0A }, /* Messwert Rücklauf Solarkreis abfragen */
    { 0xFF, 0x75, 0x50, 0x22, 0x03, 0xEC, 0x11, 0x00 }, /* Messwert Raumtemperatur mit nachkommastelle abfragen */
  };
#endif /* #ifdef AK_READ_MESS */

#ifdef TEST
  uint8_t StartDelay = 2;
  uint16_t Para = 0x1400;
  /* BM 00-1c */
#endif /* #ifdef TEST */

/****************************************************************************
* Funktionen
****************************************************************************/

/*--------------------------------------------------------------------------
* ExecCommand - Befehl ausführen
* Parameter: pBuf - Zeiger auf den Befehl
* Ergebnis:  keins
*--------------------------------------------------------------------------*/

void ExecCommand( char *pBuf )

  {
  uint8_t i, ch;
  uint16_t Slave, ParaAdr;
   int16_t ParaVal;

  if( *pBuf == '!' )
    {
    ch = pBuf[ 2 ];
    
    switch( pBuf[1] )
      {
      case 'e':           /* Maske für Daten setzen */
      case 'E':
        COMMAND_PORT.print( F("!E") );
        for( i = 0; i < NumPara; i++ )
          {
          ch = pBuf[ 2 + i ];
          if( (ch >= '0') && (ch <= '3') )
            EEPromData.Enable[ i ] = ch - '0';
          else
            break;
          COMMAND_PORT.print( (char)ch );
          }
        COMMAND_PORT.println();
        break;

      case 'm':           /* Modus einstellen */
      case 'M':
        COMMAND_PORT.print( F("!M") );
        if( (ch >= '0') && (ch <= '3') )
          {
          EEPromData.Mode = ch - '0';
          COMMAND_PORT.println( (char)ch );
          }
        else
          {
          COMMAND_PORT.print( ch );
          COMMAND_PORT.println( '?' );
          }
        break;

      case 't':           /* Zykluszeit einstellen */
      case 'T':
        EEPromData.ZyklusZeit = 0;
        for( i = 2; (pBuf[i] >= '0') && (pBuf[i] <= '9'); i++)
          EEPromData.ZyklusZeit = EEPromData.ZyklusZeit * 10 + (pBuf[i] - '0');
        EEPromData.ZyklusZeit = MAX( 10, EEPromData.ZyklusZeit ) * (1000 / MSEC_PER_TICKS);
        COMMAND_PORT.print( F("!T") );
        COMMAND_PORT.print( EEPromData.ZyklusZeit / (1000 / MSEC_PER_TICKS) );
        COMMAND_PORT.println( 's' );
        break;

      case 's':           /* Message versenden */
      case 'S':           /* Heinzungsfreigabe umschalten */
        if( (ch == '1') && (pBuf[ 3 ] >= '1') && (pBuf[ 3 ] <= '5')  )
          {
          SetupParaWrMsg( SendMsg, 0x30, 0x0112, pBuf[ 3 ] - '0' );
          doSend = 1;
          }
        else if( ch == '2' )  /* nur zum Test!! */
          {
          pBuf = GetHex( &pBuf[3], &Slave );
          if( *(pBuf++) != ' ' )
            break;
          pBuf = GetHex( pBuf, &ParaAdr );
          if( *(pBuf++) != ' ' )
            break;
          pBuf = GetDec( pBuf, &ParaVal );
          SetupParaWrMsg( SendMsg, Slave, ParaAdr, ParaVal );
          doSend = 1;
          }
        else if( ch == '3' )  /* nur zum Test!! */
            doSend = 2;
        break;
  
      case 'w':            /* EEProm Funktionen */
      case 'W':
        if( (ch == 'W') || (ch == 'w') )
          {
          EEPromData.ChkSum = CalcCheckSum( (uint8_t *)&EEPromData, sizeof( struct EEPROM_DATA ) - 1 );
          EEPROM.put( 0, EEPromData );
          COMMAND_PORT.println( F("write data to EEPROM") );
          }

        else if( (ch == 'R') || (ch == 'r') )
          {
          EEPROM.get( 0, EEPromData );    /* Konfiguration lesen */
          COMMAND_PORT.println( F("read data from EEPROM") );
          if( EEPromData.ChkSum != CalcCheckSum( (uint8_t *)&EEPromData, sizeof( struct EEPROM_DATA ) - 1 ) )
            {
            memcpy_P( &EEPromData, &DefEEProm, sizeof( struct EEPROM_DATA ) );
            COMMAND_PORT.println( F("Checksum Error, default loaded") );
            }

          }
        else if( (ch == 'D') || (ch == 'd') )
          {
          memcpy_P( &EEPromData, &DefEEProm, sizeof( struct EEPROM_DATA ) );
          COMMAND_PORT.println( F("default loaded") );
          }

        else
          {
          COMMAND_PORT.print( F("!W") );
          COMMAND_PORT.print( (char)pBuf[2] );
          COMMAND_PORT.println( '?' );
          }
        break;

      default:
        COMMAND_PORT.print( '!' );
        COMMAND_PORT.print( pBuf[ 1 ] );
        COMMAND_PORT.println( '?' );
      }      
    }
  else if( *pBuf == '?' )
    {
    switch( pBuf[1] )
      {
      case 'e':               /* Maske für Daten abfragen */
      case 'E':
        COMMAND_PORT.print( F("?E") );
        for( i = 0; i < NumPara; i++ )
          COMMAND_PORT.print( (char)(EEPromData.Enable[ i ] + '0') );
        COMMAND_PORT.println();
        break;

      case 'm':               /* Modus abfragen */
      case 'M':
        COMMAND_PORT.print( F("?M") );
        COMMAND_PORT.println( (char)(EEPromData.Mode + '0') );
        break;

      case 't':               /* Zykluszeit abfragen */
      case 'T':
        COMMAND_PORT.print( F("?T") );
        COMMAND_PORT.print( EEPromData.ZyklusZeit / (1000 / MSEC_PER_TICKS) );
        COMMAND_PORT.println( 's' );
        break;

      case 'p':               /* Daten ausgeben */
      case 'P':
        if( (pBuf[2] >= '0') && (pBuf[2] <= '9') && (pBuf[3] >= '0') && (pBuf[3] <= '9') )
          {
          Dezimalpunkt = '.';
          i = 10 * (pBuf[2] - '0') + pBuf[3] - '0';
          if( i < NumPara )
            {
            FlexPrint( (char *)HeizInfo[ i ].Text );
            FlexPrint( (char *)" " );
            PrintData( HeizInfo[ i ].Data, HeizInfo[ i ].Type );
            PrintNewLine();
            }
          Dezimalpunkt = ',';
          }
        else
          PrintAllData();
        break;

      default:
        COMMAND_PORT.print( '?' );
        COMMAND_PORT.print( pBuf[ 1 ] );
        COMMAND_PORT.println( '?' );
      }
    }
  }

/*--------------------------------------------------------------------------
* DoCommand - Befehl holen und ausführen
* Parameter: keine
* Ergebnis:  keins
*--------------------------------------------------------------------------*/

void DoCommand( void )

  {
  static uint8_t CmdBufPos = 0;
  uint8_t NewBytes;

  NewBytes = COMMAND_PORT.available();
  while( NewBytes-- > 0 )
    {
    CmdBuf[ CmdBufPos ] = COMMAND_PORT.read();
    if( logFile )
      logFile.write( CmdBuf[ CmdBufPos ] );
    if( (CmdBuf[ CmdBufPos ] == '\r') || (CmdBuf[ CmdBufPos ] == '\n') || (CmdBuf[ CmdBufPos ] == '/') )
      {
      ExecCommand( CmdBuf );
      CmdBufPos = 0; 
      }
    else if( ++CmdBufPos == CMD_BUF_SIZE )     /* Overflow */
      CmdBufPos = 0; 
    }

  logFile.flush();
  }

  
/*--------------------------------------------------------------------------
* SendChrOnBus - Zeichen auf Bus senden
* Parameter: ch - Zeichen
* Ergebnis:  true - OK
*--------------------------------------------------------------------------*/

static bool SendChrOnBus( uint8_t ch )

  {
  uint8_t rch;
  
  EBUS_PORT.write( ch );                /* Zeichen auf EBus senden */
  while( !EBUS_PORT.available() );      /* Warten bis dieses Zeichen wieder empfangen wird */
  rch = EBUS_PORT.read();               /* Zeichen lesen */
  if( ch != rch )                       /* Auf Bus-Kollision prüfen */
    {
//  if( EEPromData.Mode == MODE_HEX_INFO )  /* Debug Message */
//    COMMAND_PORT.println( F("--Kollision") );
      
    ReseveBus = 10;                     /* Nach Kollision Bus fuer andere frei lassen */
    if( (rch != 0xAA) && (rch != 0xFF) )
      Buffer[ BufPos++ ] = rch;         /* ev. 1. Zeichen einer Message, falls der andere nicht auch abbricht */
    uc_crc = crc8_calc( rch, 0, 0x9B );
    }
   
  return( ch == rch );  
  }
  

/*--------------------------------------------------------------------------
* SendChrxAAChk - Zeichen auf Bus senden und auf 0xAA prüfen
* Parameter: ch - Zeichen
*            crc - Zeiger auf Prüfsumme
* Ergebnis:  true - OK
*--------------------------------------------------------------------------*/

static bool SendChrxAAChk( uint8_t ch, uint8_t *crc )

  {
  uint8_t lch = ch;
  
  if( (ch == 0xAA) || (ch == 0xA9) )      /* Auf Sonderzeichen prüfen */
    {
    *crc = crc8_calc( 0xA9, *crc, 0x9B ); /* Checksumme berechnen */
    if( !SendChrOnBus( 0xA9 ) )           /* und senden */
       return( false );
    ch -= 0xA9;
    }
    
  *crc = crc8_calc( ch, *crc, 0x9B );     /* Checksumme berechnen */
  if( !SendChrOnBus( ch ) )               /* Zeichen senden */
    return( false );

  if( (EEPromData.Mode == MODE_HEX) || (EEPromData.Mode == MODE_HEX_INFO) )
    PrintHex( lch, 3 );                  /* Zeichen ausgeben */

  return( true );
  }


/*--------------------------------------------------------------------------
* SendMsgOnBus - Message auf Bus senden
* Parameter: keine
* Ergebnis:  keins
*--------------------------------------------------------------------------*/

void SendMsgOnBus( void )

  {
  uint8_t crc;
  uint8_t MsgLen;

/*** Prüfen ob wir wegen einer Kollision den Bus freihalten müssen ***/
  if( ReseveBus )
    {
    ReseveBus--;
    return;
    }

/*** Prüfen ob und welche Message gesendet werden soll ***/
  if( doSend == 1 )     
    {
    MsgLen = SendMsg[ 4 ] + 5;
    LIMIT( MsgLen, MAX_SPECMES_LEN+5 );
    memcpy( Buffer, SendMsg, MsgLen );
    }
  else if( doSend == 2 )
    {
    MsgLen = pgm_read_byte_near( SendMsg2 + 4 ) + 5;
    LIMIT( MsgLen, EBUS_TRA_BUF_SIZE );
    memcpy_P( Buffer, SendMsg2, MsgLen );
    }
#ifdef AK_READ_MESS                       /* Zyklisch Messages */
  else if( (doSendAuto > 0) && (doSendAuto <= MAX_READ_MES) )
    {
    MsgLen = pgm_read_byte_near( &AutoMsg[ doSendAuto - 1 ][4] ) + 5;
    LIMIT( MsgLen, EBUS_TRA_BUF_SIZE );
    memcpy_P( Buffer, &AutoMsg[ doSendAuto - 1 ][0], MsgLen );
    }
#endif /* #ifdef AK_READ_MESS */
  else
    return;

/*** Prüfen ob Bus frei ***/
  while( EBUS_PORT.available() )      /* neue Zeichen empfangen */
    {
    if( EBUS_PORT.peek() != 0xAA )    /* Zeichen ungleich 0xAA -> Bus belegt, Ende */
      return;
    else
      EBUS_PORT.read();               /* Puffer leeren */
    }

/*** Masteradresse senden ***/
  if( !SendChrOnBus( Buffer[ 0 ] ) )
    return;
  crc = crc8_calc(  Buffer[ 0 ], 0, 0x9B );
  if( (EEPromData.Mode == MODE_HEX) || (EEPromData.Mode == MODE_HEX_INFO) )
    {
    FlexPrint( (char *)"->" );
    PrintHex( Buffer[ 0 ], 3 );
    }

/*** Nachricht senden ***/
  for( BufPos = 1; BufPos < MsgLen; BufPos++ )
    if( !SendChrxAAChk( Buffer[ BufPos ], &crc ) )
      return;

/*** Checksumme senden ***/
  Buffer[ BufPos++ ] = crc;
  if( !SendChrxAAChk( crc, &crc ) )
    return;
                               
  if(  Buffer[ 1 ] != 0xFE )            /* Antwort im normalen Empfangsmodus */
    {
    uc_crc = 0;
    SendAckOnBus = 2;
    MesEnd = Buffer[4] + 5;
    MesAntEnd = 0;
    }
  else                                  /* Global call -> keine Antwort */
    {
    PrintNewLine();
    EBUS_PORT.write( 0xAA );
    }

/*** Nachricht als verschickt löschen ***/
  if( doSend == 0 )
    {
    if( doSendAuto > 0 )
      doSendAuto--;
    }
  else
    doSend = 0;
  }


/*--------------------------------------------------------------------------
* MessageEnd - Ende einer Message erreicht
* Parameter: keine
* Ergebnis:  keins
*--------------------------------------------------------------------------*/

static void MessageEnd( void )

  {
  if( BufPos > 1 )             /* Puffer leer -> Ende */
    {
    if( (EEPromData.Mode == MODE_HEX) || (EEPromData.Mode == MODE_HEX_INFO) )
      PrintNewLine();

    if( BufPos >= 6 )           /* Daten Empfangen */
      {
      if( (Buffer[ 4 ] + 6) > BufPos ) /* Längenangabe falsch */
        {
        if( EEPromData.Mode == MODE_HEX_INFO )
          COMMAND_PORT.println( F("- Error: Laenge falsch") );
        }
      else if( !RecErr )
        DataFromMes( Buffer, BufPos ); /* Message auslesen */
      }
    }

  uc_crc = 0;               /* Variablen neu initialisieren */
  BufPos = 0;
  SendAckOnBus = 0;
  RecErr = false;
#ifdef SD_LOGGING
#ifdef SD_MES_LOG
    logFile.flush();
#endif /* #ifdef SD_MES_LOG */
#endif /* #ifdef SD_LOGGING */
  }


/*--------------------------------------------------------------------------
* ReciveMess - Nachrichten empfangen
* Parameter: keine
* Ergebnis:  keins
*--------------------------------------------------------------------------*/

void ReceiveMess( void )

  {
  uint8_t ch;
  uint8_t lastCRC;

  if( EBUS_PORT.available() )                      /* Neues Zeichen empfangen? */
    {
    StartTimer( TIMER_EOC, 300/MSEC_PER_TICKS );   /* 0,3s Pause = Message End */
    ch = EBUS_PORT.read();                         /* Zeichen lesen */

/*** 0xAA -> Bus ist frei ***/
    if( ch == 0xAA ) 
      {
      MessageEnd();                                 /* Hier ist der Bus frei fuer eigene Messages */
      if( (doSend + doSendAuto) > 0 )
        SendMsgOnBus();
      return;
      }

/*** Checksumme berechnen ***/
    lastCRC = uc_crc;
    uc_crc = crc8_calc( ch, uc_crc, 0x9B );

/*** Sonderzeichen damit kein 0xAA in der Message vorkommt ***/
    if( ch == 0xA9 )                               /* Special char */
      {
      while( !EBUS_PORT.available() );
      ch = EBUS_PORT.read();                       /* 2. Zeichen lesen */
      uc_crc = crc8_calc( ch, uc_crc, 0x9B);
      ch += 0xA9;
      }

/*** Zeichen im Puffer ablegen ***/
    if( BufPos < EBUS_REC_BUF_SIZE )               /* Noch Platz im Puffer -> dann speichern */
      Buffer[ BufPos ] = ch; 
    else                                           /* Overflow */
      {
      if( EEPromData.Mode == MODE_HEX_INFO )
        COMMAND_PORT.println( F("- Error: overflow") );
      MessageEnd();
      return;
      }

/*** Zeichen ausgeben ***/
    if( (EEPromData.Mode == MODE_HEX) || (EEPromData.Mode == MODE_HEX_INFO) )
      {
      if( BufPos == 1 ) 
        PrintHex( Buffer[ 0 ], 3 );

      if( BufPos > 0 ) 
        PrintHex( ch, 3 );
      }

/*** Prüfungen je nach Position ***/
    if( BufPos == 5 )                             /* 1. Längenbyte empfangen */
      {
      if( Buffer[4] > 16 )
        {
        Buffer[4] = 16;
        COMMAND_PORT.println( F(" - Error: Laenge1") );
        }
      MesEnd = Buffer[4] + 5;
      MesAntEnd = 0;
      }
    else if( BufPos >= 5 )                        /* 1. Längenbyte empfangen */
      {
      if( BufPos == MesEnd )                      /* Checksumme */ 
        {
        if( ch != lastCRC )
          {
          RecErr = true;
          if( (EEPromData.Mode == MODE_HEX) || (EEPromData.Mode == MODE_HEX_INFO) )
            {
            PrintHex( uc_crc, 2 );
            COMMAND_PORT.println( F(" - Error: Checksumme1") );
            }
          }
        if( Buffer[ 1 ] == 0xFE )                  /* Global call hört hier auf */
          {
          BufPos++;
          MessageEnd();
          return;
          }

        uc_crc = 0;                                /* ab hier neue Checksumme für Antwort */
        }

      else if( BufPos == (MesEnd + 1) )            /* Ack vom Empfänger */ 
        {
        if( ch != 0 )
          {
          RecErr = true;
          if( (EEPromData.Mode == MODE_HEX) || (EEPromData.Mode == MODE_HEX_INFO) )
            COMMAND_PORT.println( F(" - Error: kein Ack1") );
          }
        }
          
      else if( BufPos == (MesEnd + 2) )            /* Länge der Antwort */ 
        {
        if( ch <= 16 )
          MesAntEnd = MesEnd + 3 + ch;
        else                                       /* nur Ack */
          {
          BufPos++;
          MessageEnd();
          return;
          }
        }
          
      else if( BufPos == MesAntEnd )                /* Checksumme Antwort */ 
        {
        if( ch != lastCRC )
          {
          RecErr = true;
          if( (EEPromData.Mode == MODE_HEX) || (EEPromData.Mode == MODE_HEX_INFO) )
            COMMAND_PORT.println( F(" - Error: Checksumme2") );
          }

        if( SendAckOnBus >= 1 )                    /* Antwort auf eigene Message abschliessen */
          {
          EBUS_PORT.write( 0x00 );
          if( SendAckOnBus >= 2 )
            EBUS_PORT.write( 0xAA );
          SendAckOnBus = 0;
          }
        }

      else if( BufPos == (MesAntEnd + 1) )          /* Ack */ 
        {
        if( ch != 0 )
          {
          RecErr = true;
          if( (EEPromData.Mode == MODE_HEX) || (EEPromData.Mode == MODE_HEX_INFO) )
            COMMAND_PORT.println( F(" - Error: kein Ack2") );
          }
        }
      }

    BufPos++;
    }

/*** Timeout prüfen ***/
  if( !TimerRunning( TIMER_EOC) )
    {
    MessageEnd();
    return;
    }
  }


/*--------------------------------------------------------------------------
* setup - Initialisierung
* Parameter: keine
* Ergebnis:  keins
*--------------------------------------------------------------------------*/

void setup()
  {
  InitTimer();                    /* Software Timer initialisieren */
  COMMAND_PORT.begin(19200);      /* Serielle SS zum PC initialisieren */
  EBUS_PORT.begin(2400);          /* Serielle SS zum EBUS initialisieren */
  pinMode( PORT_LedGr, OUTPUT );  /* LED als output */

  HeizInfo[ _ErtragTag ].Data.P = &WolfErtrag[0];
  HeizInfo[ _ErtragSum ].Data.P = &WolfErtrag[2];

#ifdef SD_LOGGING
  if( !SD.begin( SD_LOGGING ))
    COMMAND_PORT.println(F("Card failed, or not present"));
  else
    { 
    COMMAND_PORT.println(F("card initialized."));
#ifdef SD_MES_LOG
    logFile  = SD.open("DebugLog.txt", FILE_WRITE);
#endif /* #ifdef SD_MES_LOG */
    }
#endif /* #ifdef SD_LOGGING */
  delay(1000);              // wait for a second

  EEPROM.get( 0, EEPromData );    /* Konfiguration lesen */
  if( (EEPromData.ChkSum != CalcCheckSum( (uint8_t *)&EEPromData, sizeof( struct EEPROM_DATA ) - 1 ) )
   || (EEPromData.Stamp != 0xFACE) )
    {
    memcpy_P( &EEPromData, &DefEEProm, sizeof( struct EEPROM_DATA ) );
    EEPROM.put( 0, EEPromData );
    }
  
  DataLogMsg();
  COMMAND_PORT.println( F("EBus Monitor V0.1 fuer Wolf Heizungssystheme"));
  }


/*--------------------------------------------------------------------------
* loop - Hauptschleife
* Parameter: keine
* Ergebnis:  keins
*--------------------------------------------------------------------------*/
void loop()
  {
  ReceiveMess();  /* EBus Nachrichten Empfangen */
  DoCommand();    /* Daten an Kommandport verarbeiten */
  if( !TimerRunning( TIMER_SAVE ) )
    {
#ifdef TEST
    if( StartDelay > 0 )
      StartDelay--;
#endif /* #ifdef TEST */
    StartTimer( TIMER_SAVE, EEPromData.ZyklusZeit );
#ifdef AK_READ_MESS
    doSendAuto = MAX_READ_MES;
#endif /* #ifdef AK_READ_MESS */
    if( EEPromData.Mode == MODE_DATA )
      {
      Dezimalpunkt = '.';
      FlexPrint( (char *)"WolfDat ");
      PrintSelData( 1, SEPARATOR_CCU );
      Dezimalpunkt = ',';
      }

#ifdef SD_LOGGING
    SaveSelData();
#endif /* #ifdef SD_LOGGING */
    }
#ifdef TEST
  if( (StartDelay == 0) && (doSend == 0) && (Para < 0x1500) )
    {
    SetupParaRdMsg( SendMsg, 0x75, Para++ );
    doSend = 1;
    }
#endif /* #ifdef TEST */
  }
