/****************************************************************************
* 
* Wolf.c - Wolf spezifische Messages verarbeiten 
*
****************************************************************************/

#include <Arduino.h>
#include "EBusAna.h"
#include "Global.h"
#include "Wolf.h"

/****************************************************************************
* globale Daten
****************************************************************************/

                                                 /* Leer Message zum Setzen von Werten */
const PROGMEM uint8_t SendMsgWr[] = { 0xFF, 0x30, 0x50, 0x23, 0x09, 0x00, 0x12, 0x01, 0x05, 0x00, 0x5D, 0x01, 0x00, 0x00 };
                                                 /* Leer Message zum Abfragen von Werten */
const PROGMEM uint8_t SendMsgRd[] = { 0xFF, 0x35, 0x50, 0x22, 0x03, 0x00, 0x00, 0x00 };
uint16_t WolfErtrag[6];

/****************************************************************************
* Funktionen
****************************************************************************/

/*--------------------------------------------------------------------------
* DataFromWolfMes - Daten aus Wolf spezifischen Messages lesen 
* Parameter: pBuf - Zeiger auf Puffer
*            len - Zahl der Bytes im Puffer
* Ergebnis:  keins
*--------------------------------------------------------------------------*/

void DataFromWolfMes( uint8_t *pBuf, uint8_t len )

  {
//BM an Mischer Modul, Anfrage mit Antwort!
  if( (pBuf[0] == 0x70) && (pBuf[3] == 0x14) && (len >= 19) )
    {
    HeizInfo[ _Status2      ].Data.UB  = pBuf[ 5 ];  
    HeizInfo[ _SVorlaufTemp ].Data.UW  = word( pBuf[ 7 ], pBuf[ 8 ]);
#ifndef AK_READ_MESS
    HeizInfo[ _RaumTemp     ].Data.UW  = pBuf[ 9 ] * 10;
#endif /* #ifndef AK_READ_MESS */
    HeizInfo[ _SPufferTemp  ].Data.W   = (uint16_t)128 * pBuf[ 11 ]; 
    HeizInfo[ _MischerTemp  ].Data.UW  = *((uint16_t *)&pBuf[ 17 ]);
    HeizInfo[ _Status4      ].Data.UB  = pBuf[ 16 ]; 

    if( EEPromData.Mode == MODE_HEX_INFO )
      {
      PrintText( _SVorlaufTemp );
#ifndef AK_READ_MESS
      PrintText( _RaumTemp );
#endif /* #ifndef AK_READ_MESS */
      PrintText( _SPufferTemp );
      PrintText( _MischerTemp );
      PrintText( _Status2 );
      PrintText( _Status4 );
      PrintNewLine();
      }
    }

// Solarmodul an alle
  else if( (pBuf[3] == 0x17) && (len >= 12) )
    {
    HeizInfo[ _Status        ].Data.S.SKPumpe = pBuf[5];
    HeizInfo[ _KollektorTemp ].Data.UW  = *((uint16_t *)&pBuf[ 7 ]);
    HeizInfo[ _WWSolarTemp   ].Data.UW  = 16 * *((uint16_t *)&pBuf[ 9 ]);

    if( EEPromData.Mode == MODE_HEX_INFO )
      {
      PrintText( _Status );
      PrintText( _KollektorTemp );
      PrintText( _WWSolarTemp );
      PrintNewLine();
      }
    }

// Solarmodul an alle
  else if( (pBuf[3] == 0x18) && (len >= 17) )
    {
    HeizInfo[ _SolLeist  ].Data.UW = 16 * *((uint16_t *)&pBuf[ 5 ]);
    memcpy( &WolfErtrag[0], &pBuf[  7 ], 12 );

    if( EEPromData.Mode == MODE_HEX_INFO )
      {
      PrintText( _SolLeist );
      PrintText( _ErtragTag );
      PrintText( _ErtragSum );
      PrintNewLine();
      }
    }

#ifdef AK_READ_MESS
// Werte abfragen
  else if( (pBuf[3] == 0x22) && (len >= 14) )
    {
    if( (pBuf[6] == 0x29) && (pBuf[7] == 0x0A) )
      {
      HeizInfo[ _MMPuffer ].Data.UW  = *((uint16_t *)&pBuf[ 11 ]);
      if( EEPromData.Mode == MODE_HEX_INFO )
        {
        PrintText( _MMPuffer );
        PrintNewLine();
        }
      }

    else if( (pBuf[6] == 0x2A) && (pBuf[7] == 0x0A) )
      {
      HeizInfo[ _MMRueckl ].Data.UW  = *((uint16_t *)&pBuf[ 11 ]);
      if( EEPromData.Mode == MODE_HEX_INFO )
        {
        PrintText( _MMRueckl );
        PrintNewLine();
        }
      }

    else if( (pBuf[6] == 0x2B) && (pBuf[7] == 0x0A) )
      {
      HeizInfo[ _SMRueckl ].Data.UW  = *((uint16_t *)&pBuf[ 11 ]);
      if( EEPromData.Mode == MODE_HEX_INFO )
        {
        PrintText( _SMRueckl );
        PrintNewLine();
        }
      }

    else if( (pBuf[6] == 0x11) && (pBuf[7] == 0x00) )
      {
      HeizInfo[ _RaumTemp ].Data.UW  = *((uint16_t *)&pBuf[ 11 ]);
      if( EEPromData.Mode == MODE_HEX_INFO )
        {
        PrintText( _RaumTemp );
        PrintNewLine();
        }
      }
    }
#endif /* #ifdef AK_READ_MESS */
  }


/*--------------------------------------------------------------------------
* SetupParaWrMsg - Nachricht zum schreiben von Parameter erzeugen
* Parameter: Msg      - Hier sollen die Daten landen
*            SlaveAdr - Adresse des Slave
*            ParaAdr  - Adresse des Parameteres
*            Val      - Neuer Wert, der gesetzt werden soll
* Ergebnis:  keins
*--------------------------------------------------------------------------*/

void SetupParaWrMsg( uint8_t *Msg, uint8_t SlaveAdr, uint16_t ParaAdr, uint16_t Val )

  {
  uint8_t crc = 0;

  memcpy_P( Msg, SendMsgWr, 14 );
  Msg[ 1 ] = SlaveAdr;
  crc = crc8_calc( Msg[ 6 ] = ParaAdr & 0xFF, 0, 0x5c );
  crc = crc8_calc( Msg[ 7 ] = ParaAdr >> 8, crc, 0x5c );
  crc = crc8_calc( Msg[ 8 ] = Val & 0xFF,   crc, 0x5c );
  crc = crc8_calc( Msg[ 9 ] = Val >> 8,     crc, 0x5c );
  Msg[ 5 ] = crc;
  }


/*--------------------------------------------------------------------------
* SetupParaRdMsg - Nachricht zum lesen von Parameter erzeugen
* Parameter: Msg      - Hier sollen die Daten landen
*            SlaveAdr - Adresse des Slave
*            ParaAdr  - Adresse des Parameteres
* Ergebnis:  keins
*--------------------------------------------------------------------------*/

void SetupParaRdMsg( uint8_t *Msg, uint8_t SlaveAdr, uint16_t ParaAdr )

  {
  uint8_t crc;

  memcpy_P( Msg, SendMsgRd, 8 );
  Msg[ 1 ] = SlaveAdr;
  crc = crc8_calc( Msg[ 6 ] = ParaAdr & 0xFF, 0, 0x5c );
  crc = crc8_calc( Msg[ 7 ] = ParaAdr >> 8, crc, 0x5c );
  Msg[ 5 ] = crc;
  }

